
/* Chrysalide - Outil d'analyse de fichiers binaires
 * module.c - intégration du répertoire v7 en tant que module
 *
 * Copyright (C) 2015 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "module.h"


#include "instruction.h"
#include "processor.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Ajoute le module 'arm' au module Python.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool add_arch_arm_v7_module_to_python_module(PyObject *super)
{
    bool result;                            /* Bilan à retourner           */
    PyObject *module;                       /* Sous-module mis en place    */
    int ret;                                /* Bilan d'un appel            */

    static PyModuleDef py_chrysalide_v7_module = {

        .m_base = PyModuleDef_HEAD_INIT,

        .m_name = "pychrysalide.arch.arm.v7",
        .m_doc = "Python module for Chrysalide.arch.arm.v7",

        .m_size = -1,

    };

    result = false;

    module = PyModule_Create(&py_chrysalide_v7_module);
    if (module == NULL) return false;

    ret = PyState_AddModule(super, &py_chrysalide_v7_module);
    if (ret != 0) goto aaamtpm_exit;

    ret = _PyImport_FixupBuiltin(module, "pychrysalide.arch.arm.v7");
    if (ret != 0) goto aaamtpm_exit;

    Py_INCREF(module);
    ret = PyModule_AddObject(super, "v7", module);
    if (ret != 0) goto aaamtpm_exit;

    result = true;

    result &= register_python_armv7_instruction(module);
    result &= register_python_armv7_processor(module);

 aaamtpm_exit:

    if (!result)
    {
        printf("something went wrong in %s...\n", __FUNCTION__);
        /* ... */

    }

    return result;

}
