
/* Chrysalide - Outil d'analyse de fichiers binaires
 * block.h - prototypes pour le regroupement d'un lot d'instructions
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _DECOMP_EXPR_BLOCK_H
#define _DECOMP_EXPR_BLOCK_H


#include <glib-object.h>


#include "../instruction.h"



#define G_TYPE_EXPR_BLOCK               g_expr_block_get_type()
#define G_EXPR_BLOCK(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), g_expr_block_get_type(), GExprBlock))
#define G_IS_EXPR_BLOCK(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj), g_expr_block_get_type()))
#define G_EXPR_BLOCK_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_EXPR_BLOCK, GExprBlockClass))
#define G_IS_EXPR_BLOCK_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_EXPR_BLOCK))
#define G_EXPR_BLOCK_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_EXPR_BLOCK, GExprBlockClass))



/* Définition d'un ensemble d'instructions décompilées (instance) */
typedef struct _GExprBlock GExprBlock;

/* Définition d'un ensemble d'instructions décompilées (classe) */
typedef struct _GExprBlockClass GExprBlockClass;


/* Rendu des bordures */
typedef enum _BlockBordeBehavior
{
    BBB_AUTO,
    BBB_FORCE_OFF,
    BBB_FORCE_ON

} BlockBordeBehavior;


/* Indique le type défini pour un ensemble d'instructions décompilées. */
GType g_expr_block_get_type(void);

/* Constuit un conteneur pour diverses instructions décompilées. */
GDecInstruction *g_expr_block_new(GDecInstruction *);

/* Fournit le comportement du bloc pour le rendu de ses bords. */
BlockBordeBehavior g_expr_block_get_border_behavior(const GExprBlock *);

/* Définit le comportement du bloc pour le rendu de ses bords. */
void g_expr_block_set_border_behavior(GExprBlock *, BlockBordeBehavior);

/* Ajoute une instruction décompilée au conteneur existant. */
void g_expr_block_add_item(GExprBlock *, GDecInstruction *);

/* Fournit une instruction décompilée donnée du conteneur. */
GDecInstruction *g_expr_block_get_item(const GExprBlock *, size_t);

/* Supprime une instruction décompilée du conteneur existant. */
void g_expr_block_delete_item(GExprBlock *, GDecInstruction *);



#endif  /* _DECOMP_EXPR_BLOCK_H */
